#### Purpose:
## iMessage has always been interesting to me. If you have an Apple device that can connect
## to the Web, you can receive an iMessage. The below script interacts directly with the local (macOS)
## SQL database and uses a hefty SQL Query to return desired messages in my case.
## The idea being that we could programatically send and receive iMessages, building
## Cases for the response given for the user.

#### Value:
## This was created over a couple hours during my free time.

#### Data:
## If you're interested in running this project let me know, it never saw anything outside of 
## my machine so we're going to need to customize a few options.

require 'active_record'
require 'imessage'
require 'date'

$sender = Imessage::Sender.new

class Request < ActiveRecord::Base
  Request.establish_connection(
      :adapter => 'sqlite3',
      :database => (ENV['HOME']+'/Applications/Requests/requests.db')
  )
end

class MessageDatabase < ActiveRecord::Base
  MessageDatabase.establish_connection(
      :adapter => 'sqlite3',
      :database => (ENV['HOME']+'/Library/Messages/chat.db')
  )
end

class User < MessageDatabase

  class MessageBot

    def initialize
      MessageBot.createTables
    end

    begin
      def self.retrieveMessages

        sql = "SELECT message.text, message.account, message.date, message.date_read, message.date_delivered, chat.chat_identifier, chat.account_login, attachment.filename, attachment.mime_type, attachment.total_bytes, handle.id
             FROM message
             INNER JOIN chat_message_join ON message.ROWID = chat_message_join.message_id
             INNER JOIN chat ON chat.ROWID = chat_message_join.chat_id
             INNER JOIN chat_handle_join ON chat.ROWID = chat_handle_join.chat_id
             INNER JOIN handle ON handle.ROWID = chat_handle_join.handle_id
             LEFT JOIN message_attachment_join ON message.ROWID = message_attachment_join.message_id
             LEFT JOIN attachment ON attachment.ROWID = message_attachment_join.attachment_id
             where message.account ='e:ping@mgr.af';"

        recordsQuery = MessageDatabase.connection.execute(sql)

        recordArray = Array.new()

        recordsQuery.each do |record|
          recordHash = Hash.new()
          time = Time.at(Time.new(2001, 01, 01, 00, 00, 00, "+00:00").to_i + record['date'].to_i).to_s
          (Time.now.to_date - (Time.at(Time.new(2001, 01, 01, 00, 00, 00, "+00:00").to_i + record['date'].to_i).to_date) < 1000) ? invalid = false : invalid = true #Skips messages older than 24hrs
          next if invalid
          time.empty? ? next : recordHash[:time] = time
          recordHash[:is_invalid] = invalid
          record['text'].empty? ? next : record['text']
          recordHash[:text] = record['text'].strip
          type, address = record['account_login'].split(':')
          case type
            when 'E'
              recordHash[:message_type] = 'Email'
            else
              recordHash[:message_type] = 'Mobile'
          end
          record['chat_identifier'] == address ? next : recordHash[:chat_identifier] = record['chat_identifier'] #Ignore messages sent to ourself
          recordHash[:code] = 200
          recordHash[:executed] = false
          recordHash[:is_user] = false

          recordArray << recordHash

        end
        return recordArray
      end


      def self.createTables
        if Request.connection.data_source_exists?(:requests)
          MessageBot.insertQuery
        else
          Request.connection.create_table :requests, :force => true do |t|
            t.column :time, :string, :null => false
            t.column :text, :string, :null => false
            t.column :chat_identifier, :string, :null => false
            t.column :is_user, :boolean, :null => false
            t.column :message_type, :string, :null => false
            t.column :code, :integer, :null => false
            t.column :is_invalid, :boolean, :null => false
            t.column :executed, :boolean, :null => false
          end
        end
      end

      def self.insertQuery
        data = MessageBot.retrieveMessages
        data.each do |r|
          Request.create(r)
        end
        MessageBot.requestsForExecution
      end

      def self.requestsForExecution
        t = Time.now
        time = t.strftime("%Y-%m-%d").to_s
        params = Hash.new()
        params[:executed] = false
        p nonexecuted = Request.where("executed = ?", params[:executed])
      end

      def self.isUser

      end

    rescue Exception => e
      recordHash[:code] = 201
      recordHash[:is_valid] = false
      sender.deliver(text: "#{e} \n\n Debug logs: \n #{recordHash}", contacts: ['12398228752'])
    end

  end
    MessageBot.new
end


